C
C*****CALCULATE THE CONSTITUENT COEFFICIENTS using QUAL2E kinetics****
C
      SUBROUTINE FINK
     ?              (KK,MX,J,NEQ,PV,PT,Q,A,W,XK,S,CR,IRC,NNN,LUOUT)
C
C     + + + PURPOSE + + +
C     Calculate the constituent coefficients using QUAL2E kinetics.
C     This version allows the user to specify the constituent numbers.
C
C     + + + PARAMETERS + + +
      INCLUDE 'params.inc'
C
C     + + + PARAMETER DEFINITIONS + + +
C     NOBR   - Maximum number of branches allowed in model
C     NOSC   - Maximum number of cross sections (grids) allowed in branch
C     NOPR   - Maximum number of parcels allowed in branch
C              (NOPR should be at least 20 + 2 times NOSC)
C     NOCO   - Maximum number of constituents allowed
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   KK, MX, J, NEQ, IRC, NNN, LUOUT
      REAL      PV(NOPR), PT(NOCO,NOPR), Q, A, W, XK(NOCO,NOCO),
     $          S(NOCO), CR(NOCO,NOCO)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     KK     - Parcel number
C     MX     - Grid just upstream of parcel's upstream boundary
C     J      - Time step
C     NEQ    - Number of equations (constituents)
C     PV(K)  - Volume of parcel
C     PT(L,K)- Concentration of constituent L in parcel K, mg/l
C     Q      - Discharge, cfs or cms
C     A      - Average area of the subreach (sq m or sq ft)
C     W      - Average top width of the subreach, m or ft
C     XK(L,M)- Exchange coefficient for constituent L due to the
C              concentration of constituent M
C     S(L)   - Source flux of constituent L, per hour
C     CR(M,L)- Reference (equilibrium) concentration in equation M
C              for constituent L
C     IRC    - Code to set read for new data eact time step
C     NNN    - Branch number
C
C     + + + COMMONS + + +
      INTEGER   NXSEC(NOBR), NBRCH, IENG
      COMMON  /CMBLTM/   NXSEC, NBRCH, IENG
C
C     + + + COMMON DEFINTIONS + + +
C     IENG   - Input units: 0=metric, 1=English
C     NBRCH  - Number of branches
C     NXSEC(N) Number of subreaches in branch N
C
C     + + + SAVES + + +
      INTEGER   K2O, LFO, LT, LAG, LON, LNH, LN2, LN3, LOP, LDP,
     #          LBO, LDO
      REAL      ALGSET(NOBR,NOSC), ALPH0, ALPH1, ALPH2, ALPH3, ALPH4,
     #          ALPH5, ALPH6, A1, B1, BET1(NOBR,NOSC), BET2(NOBR,NOSC),
     #          BET3(NOBR,NOSC), BET4(NOBR,NOSC), CB(NOBR,NOSC), CKL,
     #          CKN, CKP, CK1(NOBR,NOSC), CK2(NOBR,NOSC),
     #          CK3(NOBR,NOSC), CK4(NOBR,NOSC), GROMAX, PN, RSPRT,
     #          SIG2(NOBR,NOSC), SIG3(NOBR,NOSC), SIG4(NOBR,NOSC),
     #          SIG5(NOBR,NOSC), SHAD0, SHAD1, SHAD2, SONET, TE, V
C
      SAVE      ALGSET, ALPH0, ALPH1, ALPH2, ALPH3, ALPH4, ALPH5, ALPH6,
     #          A1, B1, BET1, BET2, BET3, BET4, CB, CKL, CKN, CKP, CK1,
     #          CK2, CK3, CK4, GROMAX, K2O, LFO, PN, RSPRT, SIG2, SIG3,
     #          SIG4, SIG5, SHAD0, SHAD1, SHAD2, SONET, TE, V,
     #          LT, LAG, LON, LNH, LN2, LN3, LOP, LDP, LBO, LDO
C
C     + + + SAVE DEFINITIONS + + +
C     ALGSET(N,I) Local settling rate for algae, ft/day
C     ALPH0  - Conversion factor, ug Chlorophyll-A/mg Algae
C     ALPH1  - Fraction of algae biomass that is nitrogen, mg-N/mg-A
C     ALPH2  - Fraction of algae biomass that is phosphorus, mg-P/mg-A
C     ALPH3  - Oxygen production per unit of algae growth, mg-O/mg-A
C     ALPH4  - Oxygen uptake per unit of algae expired, mg-O/mg-A
C     ALPH5  - Oxygen uptake per unit of NH3 oxidation, mg-O/mg-N
C     ALPH6  - Oxygen uptake per unit of NO2 oxidation, mg-O/mg-N
C     A1=    - Constant in wind function [mm/day kpa] suggest 3.01
C     B1=    - Mass transfer coeff [mm/d/kpa(m/s)] suggest 1.13
C     BET1(N,I)Rate constant for biological oxidation of NH3-NO2, per day
C     BET2(N,I)Rate constant for biological oxidation of NO2-NO3, per day
C     BET3(N,I)Rate constant for ON-NH3 transformation, (per day)
C     BET4(N,I)Organic phorphorus decay rate, per day
C     CB(N,I)- Exponent on flow for Option 7 computation of K2
C     CKL    - Light constant, cal/(sq cm min) = Langley/min,
C              for LFO =1, half saturation coefficient
C              for LFO =2, intensity at 71% of the max growth rate
C              for LFO =3, light intensity at max growth rate
C     CKN    - Nitrogen half-saturation constant for algae, mg/L
C     CKP    - Phosphorus half-saturation const for algae, mg/L
C     CK1(N,I) Carbonaceous BOD decay rate, per day
C     CK2(N,I) Reaeration rate, per day for option 1,Manning's for option
C              5 or 8, coefficient on Q for option 7.
C     CK3(N,I) Carbonaceous BOD sink rate, per day
C     CK4(N,I) Benthos consumption of oxygen, mg-O/(day sq ft)
C     GROMAX - Maximum specific growth rate of algae, per day
C     K2O    - Option for K2 (reaeration coefficient)
C              1=read as input, 2=Churchill, 3=O'Connor, 4=Owens
C              5=Thackston, 6=Langbien, 7= power fnt, 8=Tsivoglou
C     LFO    - Light function option (1=half saturation,
C              2=Smith's function, 3=Steel's equation)
C     LT          Constituent number for temperataure
C     LAG         Constituent number for Algae
C     LON         Constituent number for Organic nitrogen
C     LNH         Constituent number for Amonia
C     LN2         Constituent number for Nitrite
C     LN3         Constituent number for Nitrate
C     LOP         Constituent number for Organic phosphorus
C     LDP         Constituent number for Dissolved Phosphorus
C     LBO         Constituent number for BOD
C     LDO         Constituent number for Dissolved oxygen
C     PN     - Preference factor for ammonia nitrogen (0-1.0)
C     RSPRT  - Algae respiration rate, per day
C     SHAD1  - Linear algal self shading coefficient
C     SHAD2  - Nonlinear algal self shading coefficient
C     SIG2(N,I)Benthos source rate for dissolved P, mg-P/ft2 day
C     SIG3(N,I)Benthos source rate for NH3, mg-N/ft2 day
C     SIG4(N,I)Organic nitrogen settling rate, per day
C     SIG5(N,I)Organic phorphorus settling rate, per day
C     SONET  - Light intensity, cal/(sq cm min) = Langleys/min
C     TE     - Equilibrium water temperature, degrees C
C     V      - Wind speed in meters/second
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   I, I1, IGRO, K, K1, LUQ2, N
      REAL      AA, BET1T, BET2T, BET3T, BET4T, CK1T, CORDO, EXCF,
     #          FL, FN, FNKD, FNKPV, FNKQ, FNKV, FP, F1, GRO, RSPRTT,
     #          TAB, DFT, F, PSI, SE, XKX
      CHARACTER*64 VERSN
      CHARACTER*80 BLK
C
C     + + + LOCAL DEFINITIONS + + +
C     AA     - Local variable
C     BLK    - Alpha numeric character
C     BET1T  - Temperature corrected BET1, per hour
C     BET2T  - Temperature corrected BET2, per hour
C     BET3T  - Temperature corrected BET3, per hour
C     BET4T  - Temperature corrected BET4, per hour
C     CK1T   - Temperature corrected value of CK1, per hour
C     CORDO  - Nitrification rate correction factor
C     EXCF   - Light extinction coef for algae, per ft
C     FL     - Algal growth limitation factor for light
C     FN     - Algal growth limitation factor for nitrogen
C     FNKD   - Depth in ft (A/W)
C     FNKQ   - Discharge in cu ft/s
C     FNKV   - Velocity (Q/A) in ft/s
C     FNKPV  - Liters/cu ft
C     FP     - Algal growth limitation factor for phosphorus
C     F1     - Fraction of algal nitrogen uptake from ammonia pool
C     GRO    - Specific growth rate if algae
C     IGRO   - Growth rate option (1=multiplicative, 2=limiting nutrient,
C              3=Harmonic mean)
C     RSPRTT - Temperature corrected RSPRT, per hour
C     SHAD0  - Non-algal portion of light extintion coefficient, 1/ft
C     TAB    - Absolute temperature, local variable
C
C     + + + INTRINSICS + + +
      INTRINSIC   EXP, LOG
C
C     + + + FORMATS + + +
 1000 FORMAT (A)
 1001 FORMAT (1X,A)
 6000 FORMAT(' Wind function =',F6.2,'+',F6.2,
     #       'V [mm/day KPA when V is in M/s].')
 6001 FORMAT(/,30X,'Reaction Kinetics')
 6003 FORMAT(' Chlorophyll-A conent, Algae  ALPH0=',F7.1,' ug-Chl/mg-A')
 6004 FORMAT(' Nitrogen content of algae    ALPH1=',F7.3,' mg-N/mg-A')
 6005 FORMAT(' Phosporus content of algae   ALPH2=',F7.3,' mg-P/mg-A')
 6006 FORMAT(' DO produced/ algae growth    ALPH3=',F7.2,' mg-O/mg-A')
 6007 FORMAT(' DO used/ unit algae respired ALPH4=',F7.2,' mg-O/mg-A')
 6008 FORMAT(' DO used/unt ammonia oxidized ALPH5=',F7.2,' mg-0/mg-N')
 6009 FORMAT(' DO used/unt nitrite oxidized ALPH6=',F7.2,' mg-0/mg-N')
 6010 FORMAT(' Maximum specific growth rte GROMAX=',F7.2,' per day')
 6011 FORMAT(' Respiration rate for algae   RSPRT=',F7.2,' per day')
 6012 FORMAT(' Nitrogen half-saturation const CKN=',F7.3,' mg-N/L')
 6013 FORMAT(' Phos half-saturation constant  CKP=',F7.3,' mg-P/L')
 6014 FORMAT('Light intensity @ 50% of grwth CKL=',F7.3,' cal/cm^2 min')
 6015 FORMAT('Light intensity @ 71% of grwth CKL=',F7.3,' cal/cm^2 min')
 6016 FORMAT('Light intensity at max growth  CKL=',F7.3,' cal/cm^2 min')
 6017 FORMAT('Light extinction coef =',F7.3,' +',G9.2,' Chl-A +',
     #       G9.2, ' Chl-A^2/3')
 6018 FORMAT(' Preference factor for ammonia nitrogen  PN=',F7.3)
 6019 FORMAT(' Use reaeration option                  K2O=',I7)
 6030 FORMAT(' Br  Gr  ALGSET  BET3    SIG4     BET1      SIG3       ',
     #       'BET2      BET4')
 6031 FORMAT(' Br  Gr       SIG5    SIG2     CK1      CK3        ',
     #       'CK4     CK2')
 6032 FORMAT(' Br  Gr       SIG5    SIG2     CK1      CK3        CK4')
 6033 FORMAT(' Br  Gr       SIG5    SIG2     CK1      CK3        ',
     #       'CK4     Man n  ')
 6034 FORMAT(' Br  Gr       SIG5    SIG2     CK1      CK3        ',
     #       'CK4     coef a      coef b')
 6040 FORMAT(2I4,F7.2,F9.3,F7.3,F9.3,F11.3,F9.2,F11.3)
 6041 FORMAT(2I4,F11.3,F9.3,F8.3,F10.3,F11.3,F9.4,F10.2)

C     + + + END SPECIFICATIONS + + +
C
      VERSN = '@(#)QUAL2E FINK- last modified May  9, 1996, kmf'
      VERSN = '@(#)QUAL2E FINK- last modified Oct 21, 1996. hej'
C
      IF (IRC .EQ. 1) THEN
        IF (J .EQ. 1) THEN
          LT=1
          LAG=2
          LON=3
          LNH=4
          LN2=5
          LN3=6
          LOP=7
          LDP=8
          LBO=9
          LDO=10
          LUQ2=19
          OPEN(LUQ2,FILE='qual2e.in')
          READ(LUQ2,1000) BLK
          WRITE(LUOUT,1001) BLK
          READ(LUQ2,1000) BLK
          READ(LUQ2,*)A1,B1
          WRITE(LUOUT,6000)A1,B1
          READ(LUQ2,1000) BLK
          WRITE(LUOUT,6001)
          READ(LUQ2,*)ALPH0,ALPH1,ALPH2,ALPH3,ALPH4,ALPH5,ALPH6,
     #                GROMAX,IGRO,RSPRT
          WRITE(LUOUT,6003)ALPH0
          WRITE(LUOUT,6004)ALPH1
          WRITE(LUOUT,6005)ALPH2
          WRITE(LUOUT,6006)ALPH3
          WRITE(LUOUT,6007)ALPH4
          WRITE(LUOUT,6008)ALPH5
          WRITE(LUOUT,6009)ALPH6
          WRITE(LUOUT,6010)GROMAX
          WRITE(LUOUT,6011)RSPRT
          IF(IGRO.EQ.1)WRITE(LUOUT,*)' Multiplicative growth limit'
          IF(IGRO.EQ.2)WRITE(LUOUT,*)' Nutrient limiting growth'
          IF(IGRO.EQ.3)WRITE(LUOUT,*)' Harmonic mean growth limit'
          READ(LUQ2,1000) BLK
          READ(LUQ2,*)LFO,CKL,CKN,CKP,SHAD0,SHAD1,SHAD2,PN,K2O
          IF(LFO.EQ.1)WRITE(LUOUT,6014)CKL
          IF(LFO.EQ.2)WRITE(LUOUT,6015)CKL
          IF(LFO.EQ.3)WRITE(LUOUT,6016)CKL
          WRITE(LUOUT,6012)CKN
          WRITE(LUOUT,6013)CKP
          WRITE(LUOUT,6017)SHAD0,SHAD1,SHAD2
          WRITE(LUOUT,6018)PN
          READ(LUQ2,1000) BLK
          WRITE(LUOUT,6019)K2O
          WRITE(LUOUT,6030)
          DO 1 K=1,NBRCH
            I1=NXSEC(K)-1
            DO 1 K1=1,I1
              READ(LUQ2,*)N,I,ALGSET(N,I),BET3(N,I),SIG4(N,I),
     $             BET1(N,I),SIG3(N,I),BET2(N,I),BET4(N,I)
              WRITE(LUOUT,6040)N,I,ALGSET(N,I),BET3(N,I),SIG4(N,I),
     $              BET1(N,I),SIG3(N,I),BET2(N,I),BET4(N,I)
    1     CONTINUE
          IF(K2O.EQ.1)WRITE(LUOUT,6031)
          IF(K2O.EQ.2.OR.K2O.EQ.3)WRITE(LUOUT,6032)
          IF(K2O.EQ.4.OR.K2O.EQ.6)WRITE(LUOUT,6032)
          IF(K2O.EQ.5.OR.K2O.EQ.8)WRITE (LUOUT,6033)
          IF(K2O.EQ.7)WRITE(LUOUT,6034)
          READ(LUQ2,1000) BLK
          DO 2 K=1,NBRCH
            I1=NXSEC(K)-1
            DO 2 K1=1,I1
              READ(LUQ2,*)N,I,SIG5(N,I),SIG2(N,I),CK1(N,I),CK3(N,I),
     $                    CK4(N,I),CK2(N,I),CB(N,I)
              IF(K2O.EQ.1.OR.K2O.EQ.5.OR.K2O.EQ.8)WRITE(LUOUT,6041)N,I,
     $           SIG5(N,I),SIG2(N,I),CK1(N,I),CK3(N,I),CK4(N,I),CK2(N,I)
              IF(K2O.EQ.7)WRITE(LUOUT,6041)N,I,SIG5(N,I),SIG2(N,I),
     $                      CK1(N,I),CK3(N,I),CK4(N,I),CK2(N,I),CB(N,I)
              IF(K2O.EQ.2.OR.K2O.EQ.3)WRITE(LUOUT,6041)N,I,SIG5(N,I),
     $                              SIG2(N,I),CK1(N,I),CK3(N,I),CK4(N,I)
              IF(K2O.EQ.4.OR.K2O.EQ.6)WRITE(LUOUT,6041)N,I,SIG5(N,I),
     $                              SIG2(N,I),CK1(N,I),CK3(N,I),CK4(N,I)
    2     CONTINUE
          READ(LUQ2,1000) BLK
        END IF
        READ(LUQ2,*)AA,TE,V,SONET
C       WRITE(LUOUT,*)J,TE,V,SONET
      END IF
      IF(IENG.EQ.0)THEN
        FNKD=(A/W)/0.3048
        FNKV=(Q/A)/0.3048
        FNKQ=Q/(0.3048**3.0)
      ELSE
        FNKD=A/W
        FNKQ=Q
        FNKV=Q/A
      END IF
      FNKPV=(30.48**3)/1000.0
C
C     TEMPERATURE (SIMPLIFIED TEMPERATURE ALGORITHM) (LAG)
C
      AA= 595.9-0.545*PT(LT,KK)
      PSI= (A1+B1*V)/(24.0*10.0)
      TAB= PT(LT,KK)+273.16
      DFT=1.1532E11*EXP(-4271.1/(PT(LT,KK)+242.63))
     1             /((PT(LT,KK)+242.63)**2)
      XKX= 4.0*0.97*(1.171E-7/24.0)*(TAB**3)+AA*PSI*(DFT+0.06)
      XK(LT,LT)=0.0
      IF(FNKD.GT.0.0)XK(LT,LT)= -XKX/(FNKD*30.48)
      CR(LT,LT)=TE
C
C     Algae (LAG)
C
      XK(LAG,LAG)=0.0
      IF(FNKD.GT.0.0)THEN
        FN=(PT(LNH,KK)+PT(LN3,KK))/(CKN+PT(LNH,KK)+PT(LN3,KK))
        FP=PT(LDP,KK)/(CKP+PT(LDP,KK))
        EXCF=SHAD0+SHAD1*ALPH0*PT(LAG,KK)
        IF(PT(LAG,KK).GT.0.0)
     #    EXCF=EXCF+SHAD2*((ALPH0*PT(LAG,KK))**0.66667)
        EXCF=EXCF*FNKD
        FL=1.0
        IF(EXCF.GT.0.0)THEN
          IF(LFO.EQ.1)THEN
C           Use half saturation option for light attenuation
            FL=(LOG((CKL+SONET)/(CKL+SONET*EXP(-EXCF))))/EXCF
          ELSE
            AA=SONET/CKL
            IF(LFO.EQ.2)THEN
C             Use Smith's function
              FL=(AA+(1.0+AA*AA)**0.5)
              FL=(LOG(FL/(AA*EXP(-EXCF)+(1.0+(AA*EXP(-EXCF))**2)**0.5)))
     $            /EXCF
            ELSE
C             Use Steel's function
              FL=2.718*(2.718-(EXP(-EXCF*AA))-EXP(-AA))/EXCF
            END IF
          END IF
        END IF
        IF(IGRO.EQ.1)THEN
C         Use multiplicative growth rate
          GRO=GROMAX*FL*FN*FP
        ELSE
          IF(IGRO.EQ.2)THEN
C           Use limiting nutrient growth rate
            GRO=GROMAX*FL*FN
            IF(FP.LT.FN)GRO=GROMAX*FL*FP
          ELSE
C           Use harmonic mean growth rate
            GRO=0.0
            AA=FN+FP
            IF(AA.GT.0.0)GRO=GROMAX*FL*(2.0*FN*FP/AA)
          END IF
        END IF
        GRO=(GRO*(1.047**(PT(LT,KK)-20.0)))/24.0
        AA=(ALGSET(NNN,MX)*1.024**(PT(LT,KK)-20.0))/24.0
        RSPRTT=(RSPRT*1.047**(PT(LT,KK)-20.0))/24.0
        XK(LAG,LAG)=GRO-RSPRTT
        IF(FNKD.GT.0.0)XK(LAG,LAG)=XK(LAG,LAG)-AA/FNKD
      END IF
C
C     ORGANIC NITROGEN (LON)
C
      XK(LON,LAG)=ALPH1*RSPRTT
      BET3T=(BET3(NNN,MX)*(1.047**(PT(LT,KK)-20.0)))/24.0
      XK(LON,LON)=-BET3T-(SIG4(NNN,MX)*(1.024**(PT(LT,KK)-20.0))/24.0)
C
C     AMMONIA NITROGEN (LNH)
C
      XK(LNH,LON)=BET3T
      AA=0.7
      CORDO=1.0-EXP(-AA*PT(LDO,KK))
      BET1T=BET1(NNN,MX)*(1.047**(PT(LT,KK)-20.0))/24.0
      IF(PT(LDO,KK).LE.0.0)BET1T=0.0
      XK(LNH,LNH)=-CORDO*BET1T
      AA=SIG3(NNN,MX)*(1.074**(PT(LT,KK)-20.0))
      IF(FNKD.GT.0.0)S(LNH)=AA/(FNKD*FNKPV*24.0)
      AA=PN*PT(LNH,KK)-(1.0-PN)*PT(LN3,KK)
      F1=0.5
      IF(AA.GT.0.0)F1=PN*PT(LNH,KK)/(AA)
      XK(LNH,LAG)=-GRO*ALPH1*F1
C
C     NITRITE NITROGEN (LN2)
C
      XK(LN2,LNH)=-XK(LNH,LNH)
      XK(LN2,LN2)=0.0
      BET2T=BET2(NNN,MX)*(1.047**(PT(LT,KK)-20.0))/24.0
      IF(PT(LDO,KK).LE.0.0)BET2T=0.0
      XK(LN2,LN2)=-CORDO*BET2T
C
C     NITRATE NITROGEN (LN3)
C
      XK(LN3,LAG)=-ALPH1*GRO*(1.0-F1)
      XK(LN3,LN2)=-XK(LN2,LN2)
C
C     0RGANIC PHOSPHORUS (DISSOLVED OROPHOSPHATE) (LOP)
C
      XK(LOP,LAG)=ALPH2*RSPRTT
      BET4T=BET4(NNN,MX)*(1.047**(PT(LT,KK)-20.0))/24.0
      XK(LOP,LOP)=-BET4T-SIG5(NNN,MX)*(1.024**(PT(LT,KK)-20.0))/24.0
C
C     DISSOLVED PHOSPHORUS (LDP)
C
      XK(LDP,LOP)=BET4T
      XK(LDP,LAG)=-ALPH2*GRO
      AA=SIG2(NNN,MX)*(1.074**(PT(LT,KK)-20.0))
      IF(FNKD.GT.0.0)S(LDP)=AA/(FNKD*FNKPV*24.0)
C
C     CBOD (LBO)
C
      CK1T=CK1(NNN,MX)*(1.047**(PT(LT,KK)-20.0))/24.0
      IF(PT(LDO,KK).GT.0.0)CK1T=CK1T*(1.0-EXP(-PT(LDO,KK)))
      IF(PT(LDO,KK).LE.0.0)CK1T=0.0
      XK(LBO,LBO)=-CK1T-CK3(NNN,MX)*(1.024**(PT(LT,KK)-20.0))/24.0
C
C     DISSOLVED OXYGEN (LDO)
C
      XK(LDO,LAG)=(ALPH3*GRO-ALPH4*RSPRTT)
      XK(LDO,LNH)=-ALPH5*BET1T*CORDO
      XK(LDO,LN2)=-ALPH6*BET2T*CORDO
      XK(LDO,LBO)=-CK1T
      AA=-139.3441+(1.575701E5/TAB)-(6.642308E7/(TAB*TAB))
      AA=AA+(1.2438E10/(TAB*TAB*TAB))-8.621949E11/(TAB**4.0)
      CR(LDO,LDO)=EXP(AA)
      IF(K2O.LT.5)THEN
        IF(K2O.LT.3)THEN
          IF(K2O.EQ.1)THEN
C           Read K2 from input
            XK(LDO,LDO)=CK2(NNN,MX)
          ELSE
C           Use Churchill, Elmore, & Buckingham (1962)
            XK(LDO,LDO)=5.026*(ABS(FNKV)*0.969)*(FNKD**(-1.673))*2.31
          END IF            
        ELSE
          IF(K2O.EQ.3)THEN
C           Use O'Connor & Dobbins (1958)
            AA=(1.91E3*(1.037**(PT(LT,KK)-20.0))*ABS(FNKV))**0.5
            XK(LDO,LDO)=AA/(FNKD**1.5)
          ELSE
C           Use Owens et al (1964)
            XK(LDO,LDO)=9.4*(ABS(FNKV)**0.67)*(FNKD**(-1.85))*2.31
          END IF
        END IF
      ELSE
        IF(K2O.LT.7)THEN
          IF(K2O.EQ.5)THEN
C           Use Thackston & Krenkel (1966)
C           NEEDS MANNING'S N
            AA=ABS(FNKV)*CK2(NNN,MX)*(32.2**0.5)/(1.49*FNKD**1.167)
            F=(AA/((32.2*FNKD)**0.5))**0.5
            XK(LDO,LDO)=10.8*(1.0+F)*AA*2.31/FNKD
          ELSE
C           Use Langbien & Durum (1967)
            XK(LDO,LDO)=3.3*ABS(FNKV)*(FNKD**(-1.33))*2.31
          END IF            
        ELSE
          IF(K2O.EQ.7)THEN
C           Use power function
C           needs CA AND CB
            XK(LDO,LDO)=CK2(NNN,MX)*(FNKQ**CB(NNN,MX))
          ELSE
C           Use Tsivoglou & Wallace (1972)
C           NEEDS MANNING'S N
            SE=FNKV*FNKV*(CK2(NNN,MX)**2)/(1.49*1.49*(FNKD**1.3333))
            AA=0.054
            IF(FNKQ.LT.15.0)AA=0.110
            XK(LDO,LDO)=(3600.0*24.0)*AA*SE*ABS(FNKV)
          END IF
        END IF
      END IF
      XK(LDO,LDO)=-XK(LDO,LDO)*(1.024**(PT(LT,KK)-20.0))/24.0
      S(LDO)=0.0
      AA=-CORDO*CK4(NNN,MX)*(1.06**(PT(LT,KK)-20.0))
      IF(FNKD.GT.0.0)S(LDO)=AA/(FNKD*FNKPV*24.0)
      IRC=0
C
      RETURN
      END
